def csv_colunm_foramt(val):
    if str(val).find("$") >= 0:
        return float(val.replace("$", "").replace(",", ""))
    if str(val).find("¥") >= 0:
        return float(val.replace("¥", "").replace(",", ""))
    return val



from dbgpt.model.proxy.llms.chatgpt import OpenAILLMClient
from dbgpt.model.proxy.llms.gemini import GeminiLLMClient
from dbgpt.model.proxy.llms.spark import SparkLLMClient
from dbgpt.model.proxy.llms.tongyi import TongyiLLMClient
from dbgpt.model.proxy.llms.wenxin import WenxinLLMClient
from dbgpt.model.proxy.llms.zhipu import ZhipuLLMClient

__ALL__ = [
    "OpenAILLMClient",
    "GeminiLLMClient",
    "TongyiLLMClient",
    "ZhipuLLMClient",
    "WenxinLLMClient",
    "SparkLLMClient",
]



import functools

import click


def add_tap_options(func):
    @click.option(
        "-r",
        "--repo",
        type=str,
        default=None,
        required=False,
        help="The repository to install the dbgpts from",
    )
    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        return func(*args, **kwargs)

    return wrapper


@click.command(name="install")
@add_tap_options
@click.argument("name", type=str)
def install(repo: str | None, name: str):
    """Install your dbgpts(operators,agents,workflows or apps)"""
    from .repo import install

    install(name, repo)


@click.command(name="uninstall")
@click.argument("name", type=str)
def uninstall(name: str):
    """Uninstall your dbgpts(operators,agents,workflows or apps)"""
    from .repo import uninstall

    uninstall(name)


@click.command(name="list")
def list_all_apps():
    """List all installed dbgpts"""
    from .repo import list_repo_apps

    list_repo_apps()


@click.command(name="list")
def list_repos():
    """List all repos"""
    from .repo import list_repos

    print("\n".join(list_repos()))


@click.command(name="add")
@add_tap_options
@click.option(
    "--url",
    type=str,
    required=True,
    help="The URL of the repo",
)
def add_repo(repo: str, url: str):
    """Add a new repo"""
    from .repo import add_repo

    add_repo(repo, url)


@click.command(name="remove")
@click.argument("repo", type=str)
def remove_repo(repo: str):
    """Remove the specified repo"""
    from .repo import remove_repo

    remove_repo(repo)


@click.command(name="update")
@click.option(
    "-r",
    "--repo",
    type=str,
    default=None,
    required=False,
    help="The repository to update(Default: all repos)",
)
def update_repo(repo: str | None):
    """Update the specified repo"""
    from .repo import list_repos, update_repo

    for p in list_repos():
        if repo:
            if p == repo or repo == "all":
                print(f"Updating repo '{p}'...")
                update_repo(p)

        else:
            print(f"Updating repo '{p}'...")
            update_repo(p)



"""Trigger manager for AWEL.

After DB-GPT started, the trigger manager will be initialized and register all triggers
"""
import logging
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING, Any, Dict, Optional

from dbgpt.component import BaseComponent, ComponentType, SystemApp

from .base import Trigger

if TYPE_CHECKING:
    from fastapi import APIRouter


logger = logging.getLogger(__name__)


class TriggerManager(ABC):
    """Base class for trigger manager."""

    @abstractmethod
    def register_trigger(self, trigger: Any, system_app: SystemApp) -> None:
        """Register a trigger to current manager."""

    @abstractmethod
    def unregister_trigger(self, trigger: Any, system_app: SystemApp) -> None:
        """Unregister a trigger to current manager."""

    def keep_running(self) -> bool:
        """Whether keep running.

        Returns:
            bool: Whether keep running, True means keep running, False means stop.
        """
        return False


class HttpTriggerManager(TriggerManager):
    """Http trigger manager.

    Register all http triggers to a router.
    """

    def __init__(
        self,
        router: Optional["APIRouter"] = None,
        router_prefix: str = "/api/v1/awel/trigger",
    ) -> None:
        """Initialize a HttpTriggerManager.

        Args:
            router (Optional["APIRouter"], optional): The router. Defaults to None.
            If None, will create a new FastAPI router.
            router_prefix (str, optional): The router prefix. Defaults
            to "/api/v1/awel/trigger".
        """
        if not router:
            from fastapi import APIRouter

            router = APIRouter()
        self._inited = False
        self._router_prefix = router_prefix
        self._router = router
        self._trigger_map: Dict[str, Trigger] = {}

    def register_trigger(self, trigger: Any, system_app: SystemApp) -> None:
        """Register a trigger to current manager."""
        from .http_trigger import HttpTrigger

        if not isinstance(trigger, HttpTrigger):
            raise ValueError(f"Current trigger {trigger} not an object of HttpTrigger")
        trigger_id = trigger.node_id
        if trigger_id not in self._trigger_map:
            if trigger.register_to_app():
                app = system_app.app
                if not app:
                    raise ValueError("System app not initialized")
                # Mount to app, support dynamic route.
                trigger.mount_to_app(app, self._router_prefix)
            else:
                trigger.mount_to_router(self._router)
            self._trigger_map[trigger_id] = trigger

    def unregister_trigger(self, trigger: Any, system_app: SystemApp) -> None:
        """Unregister a trigger to current manager."""
        from .http_trigger import HttpTrigger

        if not isinstance(trigger, HttpTrigger):
            raise ValueError(f"Current trigger {trigger} not an object of Trigger")
        trigger_id = trigger.node_id
        if trigger_id in self._trigger_map:
            if trigger.register_to_app():
                app = system_app.app
                if not app:
                    raise ValueError("System app not initialized")
                trigger.remove_from_app(app, self._router_prefix)
            del self._trigger_map[trigger_id]

    def _init_app(self, system_app: SystemApp):
        # if self._inited:
        #     return None
        if not self.keep_running():
            return
        logger.info(
            f"Include router {self._router} to prefix path {self._router_prefix}"
        )
        app = system_app.app
        if not app:
            raise RuntimeError("System app not initialized")
        app.include_router(self._router, prefix=self._router_prefix, tags=["AWEL"])
        # self._inited = True

    def keep_running(self) -> bool:
        """Whether keep running.

        Returns:
            bool: Whether keep running, True means keep running, False means stop.
        """
        return len(self._trigger_map) > 0


class DefaultTriggerManager(TriggerManager, BaseComponent):
    """Default trigger manager for AWEL.

    Manage all trigger managers. Just support http trigger now.
    """

    name = ComponentType.AWEL_TRIGGER_MANAGER

    def __init__(self, system_app: SystemApp | None = None):
        """Initialize a DefaultTriggerManager."""
        self.system_app = system_app
        self.http_trigger = HttpTriggerManager()
        super().__init__(None)

    def init_app(self, system_app: SystemApp):
        """Initialize the trigger manager."""
        self.system_app = system_app

    def register_trigger(self, trigger: Any, system_app: SystemApp) -> None:
        """Register a trigger to current manager."""
        from .http_trigger import HttpTrigger

        if isinstance(trigger, HttpTrigger):
            logger.info(f"Register trigger {trigger}")
            self.http_trigger.register_trigger(trigger, system_app)
        # else:
        #     raise ValueError(f"Unsupported trigger: {trigger}")

    def unregister_trigger(self, trigger: Any, system_app: SystemApp) -> None:
        """Unregister a trigger to current manager."""
        from .http_trigger import HttpTrigger

        if isinstance(trigger, HttpTrigger):
            logger.info(f"Unregister trigger {trigger}")
            self.http_trigger.unregister_trigger(trigger, system_app)
        # else:
        #     raise ValueError(f"Unsupported trigger: {trigger}")

    def after_register(self) -> None:
        """After register, init the trigger manager."""
        if self.system_app:
            self.http_trigger._init_app(self.system_app)

    def keep_running(self) -> bool:
        """Whether keep running.

        Returns:
            bool: Whether keep running, True means keep running, False means stop.
        """
        return self.http_trigger.keep_running()



try:
    from dbgpt.model.cluster.client import DefaultLLMClient
except ImportError as exc:
    # logging.warning("Can't import dbgpt.model.DefaultLLMClient")
    DefaultLLMClient = None


_exports = []
if DefaultLLMClient:
    _exports.append("DefaultLLMClient")

__ALL__ = _exports



"""Base classes for AWEL."""
from abc import ABC, abstractmethod


class Trigger(ABC):
    """Base class for trigger."""

    @abstractmethod
    async def trigger(self) -> None:
        """Trigger the workflow or a specific operation in the workflow."""
